<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              xmlns:wwexec="urn:WebWorks-XSLT-Extension-Execute"
                              xmlns:wwenv="urn:WebWorks-XSLT-Extension-Environment"
                              xmlns:wwlocale="urn:WebWorks-Locale-Schema"
                              xmlns:wwmultisere="urn:WebWorks-XSLT-Extension-MultiSearchReplace"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc wwexec wwenv wwlocale wwmultisere"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:variable name="GlobalFilenameSpacesToUnderscores" select="wwprojext:GetFormatSetting('filename-spaces-to-underscores') = 'true'" />
 <xsl:variable name="GlobalFilenameSpacesToUnderscoresSearchString">
  <xsl:if test="$GlobalFilenameSpacesToUnderscores">
   <xsl:text> </xsl:text>
  </xsl:if>
 </xsl:variable>
 <xsl:variable name="GlobalFilenameSpacesToUnderscoresReplaceString">
  <xsl:if test="$GlobalFilenameSpacesToUnderscores">
   <xsl:text>_</xsl:text>
  </xsl:if>
 </xsl:variable>

 <xsl:variable name="GlobalInvalidPathCharactersExpression" select="'[#&amp;?]'" />


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Generate WebWorks Help compatible API files? -->
   <!--                                              -->
   <xsl:variable name="VarConnectAPIWWHelpSetting" select="wwprojext:GetFormatSetting('connect-api-wwhelp')" />
   <xsl:if test="$VarConnectAPIWWHelpSetting = 'true'">
    <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
    <xsl:variable name="VarProgressGroupsAndMergeStart" select="wwprogress:Start(count($VarProjectGroups) + 1)" />

    <!-- Locate template -->
    <!--                 -->
    <xsl:variable name="VarAPITemplatePath" select="wwuri:AsFilePath('wwformat:API/wwhelp/wwhimpl/api.htm')" />

    <!-- Create entry point per group -->
    <!--                              -->
    <xsl:for-each select="$VarProjectGroups">
     <xsl:variable name="VarProjectGroup" select="." />

     <xsl:variable name="VarProgressGroupStart" select="wwprogress:Start(1)" />

     <!-- Determine group name -->
     <!--                      -->
     <xsl:variable name="VarGroupName" select="wwprojext:GetGroupName($VarProjectGroup/@GroupID)" />

     <!-- Determine group output directory path -->
     <!--                                       -->
     <xsl:variable name="VarGroupOutputDirectoryPath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), wwstring:ReplaceWithExpression(wwstring:Replace($VarGroupName, $GlobalFilenameSpacesToUnderscoresSearchString, $GlobalFilenameSpacesToUnderscoresReplaceString), $GlobalInvalidPathCharactersExpression, '_'))" />

     <!-- Up to date? -->
     <!--             -->
     <xsl:variable name="VarPath" select="wwfilesystem:Combine($VarGroupOutputDirectoryPath, 'wwhelp', 'wwhimpl', 'api.htm')" />
     <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, $GlobalProject/wwproject:Project/@ChangeID, $VarProjectGroup/@GroupID, '', $GlobalActionChecksum)" />
     <xsl:if test="not($VarUpToDate)">
      <xsl:call-template name="API-WWHelp">
       <xsl:with-param name="ParamAPITemplatePath" select="$VarAPITemplatePath" />
       <xsl:with-param name="ParamRelativeEntryPath" select="concat('../../../', wwprojext:GetFormatSetting('connect-entry'))" />
       <xsl:with-param name="ParamAPITargetPath" select="$VarPath" />
      </xsl:call-template>
     </xsl:if>

     <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="{$VarProjectGroup/@GroupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
      <wwfiles:Depends path="{$VarAPITemplatePath}" checksum="{wwfilesystem:GetChecksum($VarAPITemplatePath)}" groupID="" documentID="" />
     </wwfiles:File>

     <xsl:variable name="VarProgressGroupEnd" select="wwprogress:End()" />
    </xsl:for-each>

    <!-- Create entry point for combined output -->
    <!--                                        -->
    <xsl:variable name="VarProgressMergeStart" select="wwprogress:Start(1)" />

    <!-- Up to date? -->
    <!--             -->
    <xsl:variable name="VarMergePath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), 'wwhelp', 'wwhimpl', 'api.htm')" />
    <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarMergePath, $GlobalProject/wwproject:Project/@ChangeID, '', '', $GlobalActionChecksum)" />
    <xsl:if test="not($VarUpToDate)">
     <xsl:call-template name="API-WWHelp">
      <xsl:with-param name="ParamAPITemplatePath" select="$VarAPITemplatePath" />
      <xsl:with-param name="ParamRelativeEntryPath" select="concat('../../', wwprojext:GetFormatSetting('connect-entry'))" />
      <xsl:with-param name="ParamAPITargetPath" select="$VarMergePath" />
     </xsl:call-template>
    </xsl:if>

    <wwfiles:File path="{$VarMergePath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarMergePath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
     <wwfiles:Depends path="{$VarAPITemplatePath}" checksum="{wwfilesystem:GetChecksum($VarAPITemplatePath)}" groupID="" documentID="" />
    </wwfiles:File>

    <xsl:variable name="VarProgressMergeEnd" select="wwprogress:End()" />

    <xsl:variable name="VarProgressGroupsAndMergeEnd" select="wwprogress:End()" />
   </xsl:if>

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="API-WWHelp">
  <xsl:param name="ParamAPITemplatePath" />
  <xsl:param name="ParamRelativeEntryPath" />
  <xsl:param name="ParamAPITargetPath" />

  <xsl:variable name="VarReplacementsAsXML">
   <wwmultisere:Entry match="'../../index.html'" replacement="'{$ParamRelativeEntryPath}'" />
  </xsl:variable>
  <xsl:variable name="VarReplacements" select="msxsl:node-set($VarReplacementsAsXML)/*" />
  <xsl:value-of select="wwmultisere:ReplaceAllInFile('UTF-8', $ParamAPITemplatePath, wwprojext:GetFormatSetting('encoding', 'utf-8'), $ParamAPITargetPath, $VarReplacements)" />
 </xsl:template>
</xsl:stylesheet>

